'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash.camelcase');

var _lodash2 = _interopRequireDefault(_lodash);

var _promiseCache = require('../utils/promiseCache');

var _promiseCache2 = _interopRequireDefault(_promiseCache);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_STEPS = ['prepare', 'api', 'process'];
var DEFAULT_METHOD_CACHE_CONFIG = {
  isGlobal: false,
  enabled: false,
  ttl: 60000
};

var isFunction = function isFunction(func) {
  return func && {}.toString.call(func) === '[object Function]';
};

var getMethodCacheName = function getMethodCacheName(instance, method) {
  return instance.constructor.uid + '@' + method;
};

var buildCaches = function buildCaches(instance) {
  var caches = new Map();
  var cacheConfig = instance.constructor.cache || {};
  var defaultCacheConfig = _extends({}, DEFAULT_METHOD_CACHE_CONFIG, instance.constructor.defaultCache);

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(cacheConfig)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var method = _step.value;

      var config = _extends({}, defaultCacheConfig, cacheConfig[method]);

      var cache = null;
      var cacheMehtodName = getMethodCacheName(instance, method);

      if (config.enabled) {
        if (config.isGlobal) {
          if (!instance.constructor.caches.hasOwnProperty(cacheMehtodName)) {
            instance.constructor.caches[cacheMehtodName] = new _cache2.default(config);
          }
          cache = instance.constructor.caches[cacheMehtodName];
        } else {
          cache = new _cache2.default(config);
        }

        caches.set(cacheMehtodName, cache);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return caches;
};

var _steps = Symbol('steps');
var _caches = Symbol('caches');
var _debugLogs = Symbol('debug logs');

var uid = 1;

var Source = function () {
  function Source() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref$options = _ref.options,
        options = _ref$options === undefined ? {} : _ref$options,
        _ref$steps = _ref.steps,
        steps = _ref$steps === undefined ? DEFAULT_STEPS : _ref$steps,
        _ref$context = _ref.context,
        context = _ref$context === undefined ? {} : _ref$context,
        _ref$headers = _ref.headers,
        headers = _ref$headers === undefined ? {} : _ref$headers;

    _classCallCheck(this, Source);

    if (!this.constructor.uid) {
      this.constructor.uid = uid++;
    }

    this.context = context;
    this.headers = headers;
    this.options = options;

    this[_steps] = this.constructor.steps || steps;
    this[_caches] = buildCaches(this);

    if (this.constructor.api) {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = Object.keys(this.constructor.api)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var id = _step2.value;

          this[(0, _lodash2.default)('invoke ' + id)] = this.invoke.bind(this, id);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  }

  _createClass(Source, [{
    key: 'getSteps',
    value: function getSteps() {
      return this[_steps];
    }
  }, {
    key: 'enableDebugLogs',
    value: function enableDebugLogs() {
      this[_debugLogs] = true;
    }
  }, {
    key: 'disableDebugLogs',
    value: function disableDebugLogs() {
      this[_debugLogs] = false;
    }
  }, {
    key: 'getCache',
    value: function getCache(method) {
      var cacheMehtodName = getMethodCacheName(this, method);
      return this[_caches].get(cacheMehtodName);
    }
  }, {
    key: 'invoke',
    value: function invoke(method) {
      var _this = this;

      var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      var log = null;

      if (this[_debugLogs]) {
        log = function log() {
          var _console;

          for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          (_console = console).log.apply(_console, ['Bivrost --> '].concat(args));
        };

        log('"' + method + '" call argumengts', params, context);
      }

      var fn = function fn(params, context) {
        var stepsPromise = Promise.resolve(params);

        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          var _loop = function _loop() {
            var stepId = _step3.value;

            var step = null;

            if (isFunction(stepId)) {
              step = stepId;
            } else {
              var stepConfig = _this.constructor[stepId] || {};
              step = isFunction(stepConfig) ? stepConfig : stepConfig[method];

              if (!step) {
                return 'continue';
              }
            }

            stepsPromise = stepsPromise.then(function (input) {
              if (log) {
                log('"' + stepId + '" call', input, context);
              }

              return Promise.resolve(null).then(function () {
                return step(input, context);
              }).then(function (output) {
                if (log) {
                  log('"' + stepId + '" response', output);
                }

                return output;
              }).catch(function (error) {
                if (log) {
                  log('"' + stepId + '" error', error);
                }

                return Promise.reject(error);
              });
            });
          };

          for (var _iterator3 = _this[_steps][Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _ret = _loop();

            if (_ret === 'continue') continue;
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }

        return stepsPromise;
      };

      var cache = this.getCache(method);

      var methodContext = _extends({}, this.context, context);

      var headers = _extends({}, this.context.headers || {}, this.headers || {}, context.headers || {});

      if (Object.keys(headers).length) {
        methodContext.headers = headers;
      }

      if (!cache) {
        return fn(params, methodContext);
      } else {
        var key = this.getCacheKey(method, params, methodContext);

        if (log) {
          if (cache.has(key)) {
            log('get from cache by key "' + key + '"');
          } else {
            log('cache miss');
          }
        }

        return (0, _promiseCache2.default)(cache, key, function () {
          return fn(params, methodContext);
        });
      }
    }
  }, {
    key: 'getCacheKey',
    value: function getCacheKey(method) {
      var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      return JSON.stringify(params);
    }
  }, {
    key: 'clearCache',
    value: function clearCache(method, params) {
      var cacheKey = null;
      if (params) {
        cacheKey = this.getCacheKey(method, params);
      }

      var caches = method ? [this.getCache(method)] : this[_caches].values();

      if (this[_debugLogs]) {
        if (cacheKey) {
          console.log('Bivrost ---> clear cache for "' + method + '@' + cacheKey + '"');
        } else {
          console.log('Bivrost ---> clear all "' + method + '" caches');
        }
      }

      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = caches[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var cache = _step4.value;

          if (cache) {
            cache.clear(cacheKey);
          }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }
    }
  }]);

  return Source;
}();

Source.caches = [];
exports.default = Source;