'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = clientRequest;

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _createRequestTemplate = require('./createRequestTemplate');

var _createRequestTemplate2 = _interopRequireDefault(_createRequestTemplate);

var _promiseDeduplicator = require('../utils/promiseDeduplicator');

var _promiseDeduplicator2 = _interopRequireDefault(_promiseDeduplicator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var join = function join() {
  for (var _len = arguments.length, parts = Array(_len), _key = 0; _key < _len; _key++) {
    parts[_key] = arguments[_key];
  }

  return parts.join('/').replace(/[\/]+/g, '/').replace(/\/\?/g, '?').replace(/\/\#/g, '#').replace(/\:\//g, '://');
};

var buildUrl = function buildUrl() {
  var protocol = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'http:';
  var host = arguments[1];
  var prefix = arguments[2];
  var path = arguments[3];
  return protocol + '//' + join(host, prefix, path);
};

function clientRequest(template) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var getRequest = (0, _createRequestTemplate2.default)(template);

  var adapter = options.adapter;

  function getRequestExecuteFunction() {
    var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var request = getRequest(params);
    var url = buildUrl(options.protocol, options.host, options.prefix, request.path);

    return function executeRequest() {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          _ref$headers = _ref.headers,
          headers = _ref$headers === undefined ? {} : _ref$headers;

      var promiseCreator = function promiseCreator() {
        return adapter(url, _extends({}, request, {
          headers: _extends({}, request.headers, headers)
        }));
      };

      if (options.deduplicate && request.method === 'GET') {
        var dedupKey = JSON.stringify([request.method, url, request.query]);

        return (0, _promiseDeduplicator2.default)(dedupKey, promiseCreator);
      } else {
        return promiseCreator();
      }
    };
  }

  getRequestExecuteFunction.stringify = function (params) {
    var request = getRequest(params);

    var url = buildUrl(options.protocol, options.host, options.prefix, request.path);

    var queryString = _qs2.default.stringify(request.query);
    return '' + url + (queryString ? '?' + queryString : '');
  };

  return getRequestExecuteFunction;
}