'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = promiseDeduplicator;

var _cache = require('../data/cache');

var _cache2 = _interopRequireDefault(_cache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * If the promise with given `key` is still running, it is returned instead of creating new one.
 * Useful for HTTP request deduplication.
 *
 * let promise1 = PromiseDeduplicator('http://example.com/', http.get.bind(http, 'http://example.com/'))
 * let promise2 = PromiseDeduplicator('http://example.com/', http.get.bind(http, 'http://example.com/'))
 * promise1 === promise2;
 */
var cacheMap = new Map();

function promiseDeduplicator(key, fnCreatePromise) {
  var cached = cacheMap.get(key);

  if (!cached) {
    var promise = fnCreatePromise().then(function (result) {
      cacheMap.delete(key);

      return result;
    }, function (error) {
      cacheMap.delete(key);

      return Promise.reject(error);
    });

    cacheMap.set(key, promise);
    return promise;
  } else {
    return cached;
  }
}